@@ -35,6 +35,7 @@
 
 #include "bacon-volume.h"
 #include "sound-juicer.h"
+#include "sj-play.h"
 
 static GstElement *pipeline = NULL;
 static guint id = 0, button_change_id = 0;
@@ -64,8 +65,11 @@ select_track (void)
       gboolean do_play;
       GtkTreeIter next_iter;
 
-      gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (track_store),
-          &next_iter, NULL, seek_to_track);
+      if (!gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (track_store),
+                                          &next_iter, NULL, seek_to_track)) {
+        g_warning (G_STRLOC ": cannot get nth child");
+        return FALSE;
+      }
       gtk_tree_model_get (GTK_TREE_MODEL (track_store), &next_iter,
           COLUMN_EXTRACT, &do_play, -1);
       if (do_play)
@@ -77,13 +81,17 @@ select_track (void)
       return FALSE;
     }
   }
-
-  gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (track_store),
-      &current_iter, NULL, seek_to_track);
+  if (!gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (track_store),
+                                     &current_iter, NULL, seek_to_track)) {
+    g_warning (G_STRLOC ": cannot get nth child");
+    return FALSE;
+  }
 
   gst_element_set_state (pipeline, GST_STATE_PAUSED);
   cd = gst_bin_get_by_name_recurse_up (GST_BIN (pipeline), "cd-source");
   gst_element_seek (pipeline, 1.0, gst_format_get_by_nick ("track"), GST_SEEK_FLAG_FLUSH, GST_SEEK_TYPE_SET, seek_to_track, GST_SEEK_TYPE_NONE, -1);
+  /* get_state neccessary because of bug #326311 */
+  gst_element_get_state (pipeline, NULL, NULL, -1);
   current_track = seek_to_track;
   seek_to_track = -1;
 
@@ -100,7 +108,6 @@ play (void)
   
   gtk_widget_set_sensitive (next_menuitem, TRUE);
   gtk_widget_set_sensitive (prev_menuitem, TRUE);
-  gtk_widget_set_sensitive (reread_menuitem, FALSE);
 }
 
 /**
@@ -124,6 +131,8 @@ stop (void)
   gtk_widget_set_sensitive (next_menuitem, FALSE);
   gtk_widget_set_sensitive (prev_menuitem, FALSE);
   gtk_widget_set_sensitive (reread_menuitem, TRUE);
+  /* Call this to ensure that the UI is reset */
+  sj_main_set_title (NULL);
 }
 
 /**
@@ -152,8 +161,8 @@ cb_hop_track (GstBus *bus, GstMessage *message, gpointer user_data)
   while (next_track < tracks) {
     gboolean do_play;
 
-    gtk_tree_model_iter_nth_child (model,
-        &next_iter, NULL, next_track);
+    if (!gtk_tree_model_iter_nth_child (model, &next_iter, NULL, next_track))
+      return;
     gtk_tree_model_get (GTK_TREE_MODEL (track_store), &next_iter,
         COLUMN_EXTRACT, &do_play, -1);
     if (do_play)
@@ -269,6 +278,12 @@ cb_state (GstBus *bus, GstMessage *message, gpointer user_data)
   gst_message_parse_state_changed (message, &old_state, &new_state, NULL);
   transition = GST_STATE_TRANSITION (old_state, new_state);
 
+  /* all pipe elements will receive state transition messages,
+   * so we filter those out. This won't be neccessary after
+   * the playbin migration */
+  if ((GstElement*)GST_MESSAGE_SRC(message) != pipeline)
+    return;
+
   if (transition == GST_STATE_CHANGE_READY_TO_PAUSED) {
     char *title;
     gtk_widget_show (seek_scale);
@@ -291,6 +306,8 @@ cb_state (GstBus *bus, GstMessage *message, gpointer user_data)
     current_track = -1;
   } else if (transition == GST_STATE_CHANGE_PAUSED_TO_PLAYING) {
     gtk_button_set_label (GTK_BUTTON (play_button), GTK_STOCK_MEDIA_PAUSE);
+    if (id)
+      g_source_remove (id);
     id = g_timeout_add (100, (GSourceFunc) cb_set_time, NULL);
     if (button_change_id) {
       g_source_remove (button_change_id);
@@ -302,6 +319,8 @@ cb_state (GstBus *bus, GstMessage *message, gpointer user_data)
       id = 0;
     }
     /* otherwise button flickers on track-switch */
+    if (button_change_id)
+      g_source_remove (button_change_id);
     button_change_id =
         g_timeout_add (500, (GSourceFunc) cb_change_button, play_button);
   }
@@ -338,7 +357,7 @@ setup (GError **err)
     g_signal_connect (bus, "message::error", G_CALLBACK (cb_error), NULL);
     g_signal_connect (bus, "message::state-changed", G_CALLBACK (cb_state), NULL);
 
-    cdp = gst_element_factory_make ("cdparanoiasrc", "cd-source");
+    cdp = gst_element_factory_make (CD_SRC, "cd-source");
     if (!cdp) {
       gst_object_unref (GST_OBJECT (pipeline));
       pipeline = NULL;
@@ -350,15 +369,28 @@ setup (GError **err)
     /* TODO: will not notice drive changes, should monitor */
     g_object_set (G_OBJECT (cdp),
                   "read-speed", 2,
-                  "device", drive->device,
+                  "device", nautilus_burn_drive_get_device (drive),
                   NULL);
 
     queue = gst_element_factory_make ("queue", "queue"); g_assert (queue);
+    /* Set the buffer size to protect against underflow on busy systems */
+    g_object_set (queue,
+                  "min-threshold-time", (guint64) 200 * GST_MSECOND,
+                  "max-size-time", (guint64) 2 * GST_SECOND,
+                  NULL);
     conv = gst_element_factory_make ("audioconvert", "conv"); g_assert (conv);
     resample = gst_element_factory_make ("audioresample", "resample"); g_assert (resample);
     volume = gst_element_factory_make ("volume", "vol"); g_assert (volume);
     g_object_set (G_OBJECT (volume), "volume", vol, NULL);
-    out = gst_element_factory_make ("gconfaudiosink", "out"); g_assert (out);
+    out = gst_element_factory_make ("gconfaudiosink", "out");
+    if (!out) {
+        /* fall back to autoaudiosink */
+        out = gst_element_factory_make ("autoaudiosink", "out");
+    }
+    if (out) {
+        if (g_object_class_find_property (G_OBJECT_GET_CLASS (out), "profile"))
+	    g_object_set (G_OBJECT (out), "profile", 1, NULL);
+    }
 
     gst_bin_add_many (GST_BIN (pipeline), cdp, queue, conv, resample, volume, out, NULL);
     if (!gst_element_link_many (cdp, queue, conv, resample, volume, out, NULL))
@@ -405,11 +437,13 @@ on_play_activate (GtkWidget *button, gpointer user_data)
     play ();
   } else if (setup (&err)) {
     char *title;
-    if (current_track != -1)
-      gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (track_store),
-          &current_iter, NULL, current_track);
+    if (current_track != -1) {
+      if (!gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (track_store),
+                                          &current_iter, NULL, current_track))
+        return;
+    }
     gtk_list_store_set (track_store, &current_iter,
-        COLUMN_STATE, STATE_IDLE, -1);
+                        COLUMN_STATE, STATE_IDLE, -1);
     if (select_track ()) {
       gtk_list_store_set (track_store, &current_iter,
           COLUMN_STATE, STATE_PLAYING, -1);
@@ -419,9 +453,10 @@ on_play_activate (GtkWidget *button, gpointer user_data)
       g_free (title);
 
       play ();
-    }
-    else
+    } else {
+      g_warning (G_STRLOC ": failed to select track");
       stop ();
+    }
   } else {
     GtkWidget *dialog;
 
@@ -445,7 +480,8 @@ on_tracklist_row_activate (GtkTreeView * treeview, GtkTreePath * path,
   gint track;
 
   model = gtk_tree_view_get_model (treeview);
-  gtk_tree_model_get_iter (model, &iter, path);
+  if (!gtk_tree_model_get_iter (model, &iter, path))
+    return;
   gtk_tree_model_get (model, &iter, COLUMN_NUMBER, &track, -1);
   if (gtk_list_store_iter_is_valid (GTK_LIST_STORE (model), &current_iter))
       gtk_list_store_set (track_store, &current_iter, COLUMN_STATE, STATE_IDLE, -1);
@@ -454,9 +490,11 @@ on_tracklist_row_activate (GtkTreeView * treeview, GtkTreePath * path,
     char *title;
     seek_to_track = track - 1;
 
-    if (current_track != -1)
-      gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (track_store),
-          &current_iter, NULL, current_track);
+    if (current_track != -1) {
+      if (!gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (track_store),
+                                          &current_iter, NULL, current_track))
+        return;
+    }
 
     gtk_list_store_set (track_store, &current_iter,
         COLUMN_STATE, STATE_IDLE, -1);
@@ -501,8 +539,8 @@ on_previous_track_activate(GtkWidget *button, gpointer data)
   while (prev_track >= 0) {
     gboolean do_play;
 
-    gtk_tree_model_iter_nth_child (model,
-        &prev_iter, NULL, prev_track);
+    if (!gtk_tree_model_iter_nth_child (model, &prev_iter, NULL, prev_track))
+      return;
     gtk_tree_model_get (GTK_TREE_MODEL (track_store), &prev_iter,
         COLUMN_EXTRACT, &do_play, -1);
     if (do_play)
@@ -559,7 +597,8 @@ on_volume_changed (GtkWidget * volb, gpointer data)
 
     volume = gst_bin_get_by_name_recurse_up (GST_BIN (pipeline), "vol");
     g_object_set (G_OBJECT (volume), "volume", vol, NULL);
-  }
+  }  
+  gconf_client_set_float (gconf_client, GCONF_AUDIO_VOLUME, vol, NULL);
 }
 
 /*
@@ -599,7 +638,11 @@ on_seek_release (GtkWidget * scale, GdkEventButton * event, gpointer user_data)
   cd = gst_bin_get_by_name_recurse_up (GST_BIN (pipeline), "cd-source");
   seeking = FALSE;
 
+  /* set_state/get_state neccessary because of bug #326311 */
+  gst_element_set_state (pipeline, GST_STATE_PAUSED);
   gst_element_seek (pipeline, 1.0, GST_FORMAT_TIME, GST_SEEK_FLAG_FLUSH, GST_SEEK_TYPE_SET, slen * val, GST_SEEK_TYPE_NONE, -1);
+  gst_element_get_state (pipeline, NULL, NULL, -1);
+  gst_element_set_state (pipeline, GST_STATE_PLAYING);
 
   return FALSE;
 }
